@@ -1,10 +1,15 @@
-//! **Rustlr** is a Yacc-style parser generator in and for Rust, designed for the
-//! creation of parsers for programming language analysis.  **Version 0.2.0**
-//! introduces signficant improvements, although older parsers are still supported.
+//! Rustlr is a Yacc-style parser generator for Rust. Versions 0.2.0
+//! introduced signficant improvements, although older parsers are still
+//! supported.
+//!
+//! A [**detailed tutorial**](<https://cs.hofstra.edu/~cscccl/rustlr_project/>)
+//! is being prepared that will explain the
+//! format of grammars and how to generate and deploy parsers for several 
+//! examples.
 //!
 //! Rustlr can create LALR(1) as well as full
 //! LR(1) parsers.  It is also capable of recognizing operator precedence and
-//! associativity declarations that allows the use of some ambiguous grammars.
+//! associativity declarations that allow the use of some ambiguous grammars.
 //! Parsers also have optional access to *external state* information that allows
 //! them to recognize more than just context-free languages.  Rustlr implements
 //! methods of error recovery similar to those found in other LR generators.
@@ -30,10 +35,6 @@
 //! for binary formatted data.  Rather, it is designed to parse text, and specifically
 //! programming language syntax for compilation and analysis.
 //!
-//! A [**detailed tutorial**](<https://cs.hofstra.edu/~cscccl/rustlr_project/>)
-//! is being prepared that will explain the
-//! format of grammars and how to generate and deploy parsers for several 
-//! examples.
 //!
 //! Rustlr should be installed as an executable (cargo install rustlr).
 //! Many of the items exported by this crate are only required by the parsers
@@ -112,7 +113,7 @@ pub use zc_parser::{ZCParser,ZCRProduction};
 /// Since this grammar is small enought (requiring less than 16 LALR states), the
 /// generated parser is readable, which is appropriate for testing.  For larger
 /// grammars, the parser generator switches to a binary representation.
-pub fn rustler(grammarname:&str, option:&str) {
+fn rustler(grammarname:&str, option:&str) {
   let mut gram1 = Grammar::new();
   let grammarfile = format!("{}.grammar",&grammarname);
 
